# -*- coding: utf-8 -*-
import os
import platform
import json
#from config import configdict

configdict = {
    "yundoc": {
        "total":6,
        "checkClass": {
            "checkTime": "CheckTime", 
            "checkOsVersion": "CheckOsVersion", 
            "checkKernel": "CheckKernel", 
            "checkSelinux": "CheckSelinux", 
            "checkSysctl": "CheckSysctl", 
            "checkPort": "CheckPort", 
            #"checkNetSub": "CheckNetSub", 
            "checkNetworkDelay": "CheckNetworkDelay", 
            "checkHardware": "CheckHardware", 
            "checkUmask": "CheckUmask", 
            "checkDisk": "CheckDisk", 
            "checkDiskIO": "CheckDiskIO", 
            "checkHostip": "CheckHostip", 
            "checkGateway": "CheckGateway", 
            "checkDns": "CheckDns", 
            "CheckSecuritySoftware": "CheckSecuritySoftware",
            "CheckSystemParameter":"CheckSystemParameter"
        }, 
        "delay":{
           "six-more":5,
           "six-less":10
       },
        "disk": {
            "inode_check":20,
            "inode_useage_rate":80,
            "six-more": {
                "use": 0.8, 
                "root_size": 95, 
                "data_size": 490, 
                "RootPath": "/", 
                "root_normal": 100, 
                "data_normal": 500,
                "warn_root_size":9,
                "warn_data_size": 90,
                "warn_root_normal": 10, 
                "warn_data_normal": 100
            },
            "six-less": {
                "use": 0.8, 
                "root_size": 95, 
                "data_size": 290, 
                "RootPath": "/", 
                "root_normal": 100, 
                "data_normal": 300,
                "warn_root_size":9,
                "warn_data_size": 90,
                "warn_root_normal": 10, 
                "warn_data_normal": 100
            }
        }, 
        "memory": {
            "x86_64":{
                 "six-more":{"cpu":8,"memery":30,"normal":"8c32g"},
                 "six-less":{"cpu":16,"memery":60,"normal":"16c64g"}
                },
            "aarch64":{
                 "six-more":{"cpu":16,"memery":60,"normal":"16c64g"},
                 "six-less":{"cpu":16,"memery":60,"normal":"16c64g"}
                },
            "mips64el":{
                 "six-more":{"cpu":16,"memery":60,"normal":"16c64g"},
                 "six-less":{"cpu":16,"memery":60,"normal":"16c64g"}
                }
        }, 
        "kernel": {
            "maxKernel": "4.20.0", 
            "minKernel": "3.10.0"
        }, 
        "diskio": {
            "six-more": 10, 
            "warn_six-more":7,
            "six-less": 10,
            "warn_six-less":5
        }, 
        "os": {
            "Ubuntu": [
                "Ubuntu 16.04",
                "Ubuntu 18.04"
            ], 
            "Red Hat": [
                "Red Hat Enterprise Linux Server release 7.4",
                "Red Hat Enterprise Linux Server release 7.9"
            ], 
            "centos": [
                "CentOS 7.3", 
                "CentOS 7.4",
                "CentOS 7.5",
                "CentOS 7.6",
                "CentOS 7.7",
                "CentOS 7.8",
                "CentOS 7.9"
            ], 
            "SUSE": [
                "SUSE Linux Enterprise Server 12 SP2", 
                "SUSE Linux Enterprise Server 12 SP3",
                "SUSE Linux Enterprise Server 12 SP5"
            ], 
            "Kylin": [
                "Kylin Linux Advanced Server V10 (Tercel)"
            ]
        }, 
        "port": {
            "redis": ["8531", "8532","8533","8534","8535","8536"], 
            "mysql": ["3306","13306"], 
            "app": ["80", "443","8203","8208","8209"], 
            "k8s": ["6443", "8443","2379-2380","10250-10252","10255","5000","5002","8472"], 
            "kubewps": ["8081", "4306","9100-9129","9090","8910-8912","9093","9094","3100"]
        },  
        "softwares": ["qaxsafed","secdog","YDservice","Symantec"]
    }, 
    "opendoc": {
        "total":6,
        "checkClass": {
            "checkTime": "CheckTime", 
            "checkOsVersion": "CheckOsVersion", 
            "checkKernel": "CheckKernel", 
            "checkSelinux": "CheckSelinux", 
            "checkSysctl": "CheckSysctl", 
            "checkPort": "CheckPort", 
            #"checkNetSub": "CheckNetSub", 
            "checkNetworkDelay": "CheckNetworkDelay", 
            "checkHardware": "CheckHardware", 
            "checkUmask": "CheckUmask", 
            "checkDisk": "CheckDisk", 
            "checkDiskIO": "CheckDiskIO", 
            "checkHostip": "CheckHostip", 
            "checkGateway": "CheckGateway", 
            "checkDns": "CheckDns", 
            "CheckSecuritySoftware": "CheckSecuritySoftware",
            "CheckSystemParameter":"CheckSystemParameter"
        }, 
        "delay":{
           "six-more":5,
           "six-less":10
       },
        "disk": {
            "inode_check":20,
            "inode_useage_rate":80,
            "six-more": {
                "use": 0.8, 
                "root_size": 95, 
                "data_size": 490, 
                "RootPath": "/", 
                "root_normal": 100, 
                "data_normal": 500,
                "warn_root_size":9,
                "warn_data_size": 90,
                "warn_root_normal": 10, 
                "warn_data_normal": 100
            },
            "six-less": {
                "use": 0.8, 
                "root_size": 95, 
                "data_size": 290, 
                "RootPath": "/", 
                "root_normal": 100, 
                "data_normal": 300,
                "warn_root_size":9,
                "warn_data_size": 90,
                "warn_root_normal": 10, 
                "warn_data_normal": 100
            }
        }, 
        "memory": {
            "x86_64":{
                 "six-more":{"cpu":8,"memery":30,"normal":"8c32g"},
                 "six-less":{"cpu":8,"memery":30,"normal":"8c32g"}
                },
            "aarch64":{
                 "six-more":{"cpu":16,"memery":30,"normal":"8c32g"},
                 "six-less":{"cpu":16,"memery":30,"normal":"16c32g"}
                },
            "mips64el":{
                 "six-more":{"cpu":16,"memery":30,"normal":"16c32g"},
                 "six-less":{"cpu":16,"memery":30,"normal":"16c32g"}
                }
        }, 
        "kernel": {
            "maxKernel": "4.20.0", 
            "minKernel": "3.10.0"
        }, 
        "diskio": {
            "six-more": 10, 
            "warn_six-more":7,
            "six-less": 10,
            "warn_six-less":5
        }, 
        "os": {
            "Ubuntu": [
                "Ubuntu 16.04",
                "Ubuntu 18.04"
            ], 
            "Red Hat": [
                "Red Hat Enterprise Linux Server release 7.4",
                "Red Hat Enterprise Linux Server release 7.9"
            ], 
            "centos": [
                "CentOS 7.3", 
                "CentOS 7.4",
                "CentOS 7.5",
                "CentOS 7.6",
                "CentOS 7.7",
                "CentOS 7.8",
                "CentOS 7.9"
            ],
            "SUSE": [
                "SUSE Linux Enterprise Server 12 SP2", 
                "SUSE Linux Enterprise Server 12 SP3",
                "SUSE Linux Enterprise Server 12 SP5"
            ], 
            "Kylin": [
                "Kylin Linux Advanced Server V10 (Tercel)"
            ]
        }, 
        "port": {
            "redis": ["8531", "8532","8533","8534","8535","8536"], 
            "mysql": ["3306","13306"], 
            "app": ["80", "443","8203","8208","8209"], 
            "k8s": ["6443", "8443","2379-2380","10250-10252","10255","5000","5002","8472"], 
            "kubewps": ["8081", "4306","9100-9129","9090","8910-8912","9093","9094","3100"]
        }, 
        "softwares": ["qaxsafed","secdog","YDservice","Symantec"]
    }, 
    "pubwps": {
        "total":2,
        "checkClass": {
            "checkTime": "CheckTime", 
            "checkOsVersion": "CheckOsVersion", 
            "checkKernel": "CheckKernel", 
            "checkSelinux": "CheckSelinux", 
            "checkSysctl": "CheckSysctl", 
            "checkPort": "CheckPort", 
            #"checkNetSub": "CheckNetSub", 
            "checkNetworkDelay": "CheckNetworkDelay", 
            "checkHardware": "CheckHardware", 
            "checkUmask": "CheckUmask", 
            "checkDisk": "CheckDisk", 
            "checkDiskIO": "CheckDiskIO", 
            "checkHostip": "CheckHostip", 
            "checkGateway": "CheckGateway",  
            "checkDns": "CheckDns", 
            "CheckSecuritySoftware": "CheckSecuritySoftware",
            "CheckSystemParameter":"CheckSystemParameter"
        }, 
        "delay":{
           "six-more":5,
           "six-less":10
       },
        "disk": {
            "inode_check":20,
            "inode_useage_rate":80,
            "six-more": {
                "use": 0.8, 
                "root_size": 95, 
                "data_size": 490, 
                "RootPath": "/", 
                "root_normal": 100, 
                "data_normal": 500,
                "warn_root_size":9,
                "warn_data_size": 90,
                "warn_root_normal": 10, 
                "warn_data_normal": 100
            },
            "six-less": {
                "use": 0.8, 
                "root_size": 95, 
                "data_size": 290, 
                "RootPath": "/", 
                "root_normal": 100, 
                "data_normal": 300,
                "warn_root_size":9,
                "warn_data_size": 90,
                "warn_root_normal": 10, 
                "warn_data_normal": 100
            }
        }, 
        "memory": {
            "x86_64":{
                 "six-more":{"cpu":8,"memery":30,"normal":"8c32g"},
                 "six-less":{"cpu":8,"memery":30,"normal":"8c32g"}
                },
            "aarch64":{
                 "six-more":{"cpu":16,"memery":30,"normal":"16c32g"},
                 "six-less":{"cpu":16,"memery":30,"normal":"16c32g"}
                },
            "mips64el":{
                 "six-more":{"cpu":16,"memery":30,"normal":"16c32g"},
                 "six-less":{"cpu":16,"memery":30,"normal":"16c32g"}
                }
        }, 
        "kernel": {
            "maxKernel": "4.20.0", 
            "minKernel": "3.10.0"
        }, 
        "diskio": {
            "six-more": 10, 
            "warn_six-more":7,
            "six-less": 10,
            "warn_six-less":5
        }, 
        "os": {
            "Ubuntu": [
                "Ubuntu 16.04",
                "Ubuntu 18.04"
            ], 
            "Red Hat": [
                "Red Hat Enterprise Linux Server release 7.4",
                "Red Hat Enterprise Linux Server release 7.9"
            ], 
            "centos":[
                "CentOS 7.3", 
                "CentOS 7.4",
                "CentOS 7.5",
                "CentOS 7.6",
                "CentOS 7.7",
                "CentOS 7.8",
                "CentOS 7.9"
            ],
            "SUSE": [
                "SUSE Linux Enterprise Server 12 SP2", 
                "SUSE Linux Enterprise Server 12 SP3",
                "SUSE Linux Enterprise Server 12 SP5"
            ], 
            "Kylin": [
                "Kylin Linux Advanced Server V10 (Tercel)"
            ]
        }, 
        "port": {
            "redis": ["8531", "8532","8533","8534","8535","8536"], 
            "mysql": ["3306","13306"], 
            "app": ["80", "443","8203","8208","8209"], 
            "k8s": ["6443", "8443","2379-2380","10250-10252","10255","5000","5002","8472"], 
            "kubewps": ["8081", "4306","9100-9129","9090","8910-8912","9093","9094","3100"]
        }, 
        "softwares": ["qaxsafed","secdog","YDservice","Symantec"]
    }
}

def osType():
    osType = platform.system()
    if osType == 'Windows' or osType == 'Darwin':
        return osType
    elif osType == 'Linux':
        output1 = os.popen('lsb_release -a')
        output1 = output1.read()
        output2 = os.popen('cat /etc/os-release')
        output2 = output2.read()
        output3 = os.popen('cat /etc/issue')
        output3 = output3.read()
        if (('Ubuntu' in output1) or ('Ubuntu' in output2) or ('Ubuntu' in output3)):
            return "Ubuntu"
        elif (('CentOS' in output1) or ('CentOS' in output2) or ('CentOS' in output3)):
            return "CentOS"
        elif (('Red Hat' in output1) or ('Red Hat' in output2) or ('Red Hat' in output3)):
            return "Red Hat"
        elif (('SUSE' in output1) or ('SUSE' in output2) or ('SUSE' in output3)):
            return "SUSE"
        elif (('Kylin' in output1) or ('Kylin' in output2) or ('Kylin' in output3)):
            if (('NeoKylin' in output1) or ('NeoKylin' in output2) or ('NeoKylin' in output3)):
                return "neokylin"
            elif (('V10' in output1) or ('V10' in output2) or ('V10' in output3)):
                return "Kylin-V10"
            else:
                return "Kylin"
        elif (('NFS' in output1) or ('NFS' in output2) or ('NFS' in output3)):
            return "NFS"
        elif (('Deepin' in output1) or ('Deepin' in output2) or ('Deepin' in output3)):
            return "Deepin"
        else:
            return "other"
checkInfo = {
    "checktime":"检查时间",
    "checkosversion": "检查操作系统版本",
    "checkkernel": "检查系统内核",
    "checkselinux": "检查Selinux",
    "checksysctl": "检查内核参数",
    "checkport": "检查端口占用",
    "checknetsub": "检查网段占用",
    "checknetworkdelay": "检查主机网络延迟",
    "checkhardware": "检查cpu内存",
    "checkumask": "检查umask",
    "checkdisk": "检测硬盘空间及挂载",
    "checkdiskio": "检查磁盘IO",
    "checkhostip":"检查主机ip",
    "checkgateway":"检查网关",
    "checksudo": "检查用户是否设置sudo",
    "checkdns":"检查主机DNS",
    "checksecuritysoftware":"检查安全软件进程",
    "checkuser":"检查用户一致性",
    "checkbash":"检查bash执行环境",
    "checkdocker":"检查用户docker",
    "checkmachinecode":"获取机器码",
    "checksystemparameter":"检查系统参数"
}

suggest_osdict = {
    "Ubuntu":"Ubuntu操作系统环境修复FAQ",
    "CentOS":"CentOS操作系统环境修复FAQ",
    "SUSE":"SUSE操作系统环境修复FAQ",
    "Red Hat":"Red Hat操作系统环境修复FAQ",
    "Kylin-V10":"Kylin-V10操作系统环境修复FAQ",
    "Kylin":"Kylin操作系统环境修复FAQ"
}
