# -*- coding: utf-8 -*-
from __future__ import division
import os, sys, imp, json,time
import collections
from util import printtable
import logging
import parser
import argparse
import datetime
import textwrap
import multiprocessing as mp
import getpass


py3k = sys.version_info[0] >= 3
if py3k:
    import subprocess
    shellcommand = subprocess
    pycommand = "python3"
else:
    import commands
    shellcommand = commands
    pycommand = "python"
from __init__ import checkInfo

check_env_path = os.path.dirname(os.path.abspath(__file__))

checkClass = collections.OrderedDict()
checklist = [
    'CheckTime',
    'CheckSudo',
    'CheckOsVersion',
    'CheckKernel',
    'CheckSelinux',
    'CheckSysctl',
    'CheckPort',
    'CheckSSH',
    'CheckNetworkDelay',
    'CheckHardware',
    'CheckUmask',
    'CheckDisk',
    'CheckDiskIO',
    'CheckHostip',
    'CheckGateway',
    'CheckDns',
    'CheckSecuritySoftware',
    "CheckBash",
    "CheckDocker",
    "CheckMachineCode"]

for k in checklist:
    checkClass[k] = k
# 进度条
def process_bar(percent, start_str='', end_str='100%', total_length=0):
    bar = ''.join(["\033[42;33m%s\033[0m"%'   '] * int(percent * total_length)) + ''
    bar = '\r' + start_str + bar.ljust(total_length) + ' {:0>4.1f}%|'.format(percent*100) + end_str
    #print (bar, end='', flush=True)
    sys.stdout.write(bar)
    sys.stdout.flush()



def get_configdict():
    try:
        return json.loads(open('config.conf').read())
    except:
        return {}

remote_tmp_dir = '/tmp/.ansible-env-test'
def remote_sshpass_scp(ip,user,password,ssh_port=22,src_path = '',remote_path=''):
    scp_cmd = "sudo scp -P {} -o 'StrictHostKeyChecking=no' -r {} {}@{}:{}".format(ssh_port,src_path,user,ip,remote_path)
    #print (scp_cmd)
    cmd = "./sshpass -p '{}' {}".format(password,scp_cmd)
    #print (cmd)
    if os.system(cmd):
        return False
    return True
    

def remote_sshpass_exec(ip,user,password,ssh_port=22,cmd_args='',popen=False):
    cmd = '''./sshpass -p '{}' ssh -p {} -o StrictHostKeyChecking=no {}@{} "{}"'''.format(password,ssh_port,user,ip,cmd_args)
    #print (cmd)
    if popen:
        res = os.popen(cmd).read().strip('\n')
        return res
    else:
        res = os.system(cmd)
        if res:
            return False
        return True

def ssh_exec(ip,user,password,ssh_port):
    #验证机器连通性
    if not remote_sshpass_exec(ip,user,password,ssh_port,'date '):
        print ("{} 机器连通性异常,请检查机器".format(ip))
        return False
    else:
        print ('{}连通性测试ok'.format(ip))
    sudo_cmd = '''echo '{}' | sudo -S tail -1 /etc/sudoers | grep "{}.*ALL.*ALL" &>/dev/null || echo '{}' | sudo -S sed -i '\$ a\\{}     ALL=\(ALL\)       NOPASSWD: ALL' /etc/sudoers'''.format(password,user,password,user)
    #print (sudo_cmd)
    if user != 'root':
        if not remote_sshpass_exec(ip,user,password,ssh_port,sudo_cmd):
            print ('设置sudo失败')
            return False
    # 创建临时目录
    src_path,remote_path = check_env_path,remote_tmp_dir
    if not remote_sshpass_exec(ip,user,password,ssh_port,'mkdir -p {}'.format(remote_path)):
        print ("创建临时目录{}失败".format(remote_path))
        return False
    if not remote_sshpass_scp(ip,user,password,ssh_port,src_path,remote_path):
        print ("拷贝检查脚本失败")
        return False
    
    python_cmd = '''if [ ! -f /usr/bin/python ];then echo {} | sudo -S ln -sv /usr/bin/python3 /usr/bin/python; fi'''.format(password)
    if not remote_sshpass_exec(ip,user,password,ssh_port,python_cmd):
        print ("设置python软链失败")
        return False
    return True

def get_check_class_files():
    check_class_files = collections.OrderedDict()
    for key, value in checkClass.items():
        script_file = "{}/{}.py".format(check_env_path, value)
        check_class_files[key] = script_file
    return check_class_files

def local_exec_script(script_file, cmd_args=None):
    if cmd_args:
        if ' ' in script_file:
            script_file=script_file.replace(' ','\ ')
        cmd = "{0} {1} {2} 2>/dev/null".format(pycommand, script_file, cmd_args)
    else:
        if ' ' in script_file:
            script_file=script_file.replace(' ','\ ')
        cmd = "{0} {1} 2>/dev/null".format(script_file, pycommand)
    #print (cmd)
    status, result = shellcommand.getstatusoutput(cmd)
    if status != 0:
        return {}
    try:
        result_dict = json.loads(result)
    except:
        return {}
    return result_dict

def ssh_host_check(host,env,data_dir,username,password,ssh_port=22,iplist='',app='yundoc',share_result={},share_lock=None):
    check_dict = get_check_class_files()
    infolist = []
    i = 1
    for key, script_file in check_dict.items():
        if os.path.exists(script_file):
            #print ("{} 正在执行[{}]....".format(host,checkInfo.get(key.lower())))
            args = "{} {} {} {} {} {} 00000 {}".format(host,env,data_dir,username,int(time.time()),','.join(iplist),app)
            if host == '127.0.0.1':
                script_result = local_exec_script(script_file, cmd_args=args)
            else:
                remote_setup_py_file = "{}/checkEnv/{}".format(remote_tmp_dir, script_file.split('checkEnv')[-1].strip('\/'))
                
                if ' ' in script_file:
                    script_file=script_file.replace(' ','\ ')
                cmd = "sudo python -B {0} {1} 2>/dev/null".format(remote_setup_py_file, args)
                script_result1 = remote_sshpass_exec(host,username,password,ssh_port,cmd,True)
                try:
                    script_result = json.loads(script_result1)
                except:
                    script_result = {}
            infolist.append(script_result)
        process_bar(i/len(check_dict.items()), start_str=host+' ', end_str='100%', total_length=len(check_dict.items()))
        i += 1
    share_lock.acquire()
    share_result[host] = infolist
    share_lock.release()
        
    return infolist

def host_check():
    os.system('sudo rm -fr __pycache__/')
    parser = argparse.ArgumentParser()
    
    parser.add_argument('-i',metavar='ip', default= '127.0.0.1',help='要检查的主机ip')
    parser.add_argument('-P',metavar='port', default= 22,help='主机端口,远程主机需填写')
    parser.add_argument('-a', choices=["yundoc", "opendoc","pubwps"],default= 'yundoc',help='应用名称')
    parser.add_argument('-d', metavar='data_dir',default= '/data',help='主机数据盘路径')
    #parser.add_argument('-env', choices=["six-more","six-less"],default="six-less",help='检查环境')
    parser.add_argument('-u', metavar='user',default= '',help='主机用户名')
    parser.add_argument('-p', metavar='password',default= 'test',help='主机密码,远程主机需填写')
    parser.add_argument('-f',action='store_true', default=False, help='读取配置文件config.conf中主机配置')
    args = parser.parse_args() 
    
    ipinfo = []
    if args.f:
        configdict = get_configdict()
        if not configdict:
            print ("请检查cnofig.conf文件数据是否为json格式")
            return False
        iplist = []
        cmd_args = []
        env = 'six-less'
        data_dir = configdict.get("data_dir",'') if configdict.get("data_dir",'') else args.d
        app = configdict.get("app",'') if configdict.get("app",'') else args.a
        for _ips in configdict.get('host_list'):
            host,port,username,password = _ips.get("ip",""),_ips.get("ssh_port",22),_ips.get("username","wps"),_ips.get("password","")
            ipinfo.append((host,port,username,password))
            iplist.append(host)
        if len(iplist) >= 6:
            env = 'six-more'
    else:  
        if not args.u:
            username = getpass.getuser()
        else:
            username = args.u
        app,host,port,password,env,data_dir = args.a,args.i,args.P,args.p,'six-less',args.d
        iplist = [host]
        ipinfo = [(host,port,username,password)]
    
    check_dict = get_check_class_files()
    messageTable=printtable.PrettyTable()
    messageTable.field_names = ["IP","检查项","详细信息","状态","处理建议"]
    if '127.0.0.1' in iplist and len(iplist) == 1:
        print ("检查应用名称: {},本地主机:{},场景:{},数据盘目录:{}.\npython run_envcheck.py -h查看帮助信息,根据参数指定参数".format(app,host,env,data_dir))
    else:
        print ("检查远程主机:{},场景:{},数据盘目录:{}.\npython run_envcheck.py -h查看帮助信息,根据参数指定参数".format(','.join(iplist),env,data_dir))
    
    share_result = mp.Manager().dict()
    share_lock = mp.Manager().Lock()
    ps = []
    for info in ipinfo:
        host,port,user,password = info
        if host != '127.0.0.1':
            res = ssh_exec(host,user,password,port)
            if not res:
                print ("\033[0;31m 服务器{} 远程异常,请检查服务器,踢出当前检查队列\033[0m".format(host))
                ipinfo.remove(info)
                continue
    for info in ipinfo:
        host,port,user,password = info
        try:
            p = mp.Process(target=ssh_host_check,args=(host,env,data_dir,user,password,port,iplist,app,share_result,share_lock,))
            ps.append(p)
            p.start()
        except Exception as err:
            print (err)
            
    for p1 in ps:
        p1.join()
    i = 1
    for ip,script_results in share_result.items():
        for script_result in script_results:
            if script_result:
                status = script_result.get('status', 2)
                if status == 1:
                    if script_result.get('suggest',''):
                        messageTable.add_row([ip, script_result.get('name'), textwrap.fill(script_result.get('info', '').strip(),45), "\033[1;33;40m警告\033[0m",textwrap.fill(script_result.get('suggest'),45)])
                    else:
                        messageTable.add_row([ip, script_result.get('name'), textwrap.fill(script_result.get('info', '').strip(),45), "\033[1;32;40m通过\033[0m",textwrap.fill(script_result.get('suggest'),45)])
                else:
                    messageTable.add_row([ip, script_result.get('name'), textwrap.fill(script_result.get('info', '').strip(),45), "\033[1;31;40m不通过\033[0m",textwrap.fill(script_result.get('suggest'),45)])
            else:
                messageTable.add_row([ip, "", "检查失败", "\033[1;31;40m不通过\033[0m",script_result.get('suggest')])
        if i < len(share_result):    
            messageTable.add_row(['===', '===', '===', '===','==='])
        i += 1
    
    messageTable.align[u"检查项"]='l'
    messageTable.align[u"详细信息"]='l'
    messageTable.align[u"处理建议"]='l'
    time_now = str(datetime.datetime.now())
    _now = time_now.split('.')[0].replace(':','_').replace(' ','_')
    filename = '/tmp/check_result_{}.log'.format(_now)
    print ('\n')
    print(messageTable)
    deploy_ids = []
    print ('检查结果保存在{}文件'.format(filename))
    for info in ipinfo:
        host,port,user,password = info
        if host != '127.0.0.1':
            id_res = remote_sshpass_exec(host,user,password,port,'sudo /tmp/.ansible-env-test/checkEnv/get_deploy_id -dir {}'.format(data_dir),True)
            deploy_ids.append(id_res)
        else:
            os.system('sudo chmod 777 get_deploy_id')
            id_res = os.popen('sudo ./get_deploy_id -dir {}'.format(data_dir)).read().strip('\n')
            deploy_ids.append(id_res)
    print ('机器码:  {}'.format(';'.join(deploy_ids)))
    with open(filename,'w') as fp:
        fp.write(str(messageTable))
        fp.write('\n')
        fp.write('机器码:  {}'.format(';'.join(deploy_ids)))
        fp.write('\n')

if __name__ == "__main__":
    os.system('clear')
    os.system("sudo chmod 777 sshpass")
    print (" _          _     ")
    print ("| | ___   _| |__   _____      ___ __  ___ ")
    print ("| |/ / | | | '_ \ / _ \ \ /\ / / '_ \/ __|")
    print ("|   <| |_| | |_) |  __/\ V  V /| |_) \__\\")
    print ("|_|\_\__.__|_.__/ \___| \_/\_/ | .__/|___/")
    print ("                               |_|   ")
    print('\033[0;32m# 该工具为独立环境检查工具,支持多机远程执行,支持命令配置和文件配置')
    print('#\033[0;32m 检查本机: python run_envcheck.py')
    print('#\033[0;32m 检查远程单机: python run_envcheck.py -i 10.13.83.117 -P 22 -u wps -p xxxx  ,app,data_dir为可选参数,不指定则为默认值')
    print('#\033[0;32m 检查远程多机: python run_envcheck.py -f  ,读取config.conf配置文件里的主机配置进行检查\033[0m')
    print ("----"*35)
    host_check()





